@@ -932,7 +932,7 @@ child_setup (gpointer user_data)
 
 static void
 notify_desktop_launch (GDBusConnection  *session_bus,
-		       const char       *desktop_file, /* filename */
+		       GDesktopAppInfo  *info,
 		       long              pid,
 		       const char       *display,
 		       const char       *sn_id,
@@ -942,6 +942,8 @@ notify_desktop_launch (GDBusConnection  *session_bus,
   GVariantBuilder uri_variant;
   GVariantBuilder extras_variant;
   GList *iter;
+  const char *desktop_file_id;
+  const char *gio_desktop_file;
 
   if (session_bus == NULL)
     return;
@@ -956,12 +958,32 @@ notify_desktop_launch (GDBusConnection  *session_bus,
 			   "startup-id",
 			   g_variant_new ("s",
 					  sn_id));
+  gio_desktop_file = g_getenv ("GIO_LAUNCHED_DESKTOP_FILE");
+  if (gio_desktop_file != NULL)
+    g_variant_builder_add (&extras_variant, "{sv}",
+			   "origin-desktop-file",
+			   g_variant_new_bytestring (gio_desktop_file));
+  if (g_get_prgname () != NULL)
+    g_variant_builder_add (&extras_variant, "{sv}",
+			   "origin-prgname",
+			   g_variant_new_bytestring (g_get_prgname ()));
+  g_variant_builder_add (&extras_variant, "{sv}",
+			 "origin-pid",
+			 g_variant_new ("x",
+					(gint64)getpid ()));
+
+  if (info->filename)
+    desktop_file_id = info->filename;
+  else if (info->desktop_id)
+    desktop_file_id = info->desktop_id;
+  else
+    desktop_file_id = "";
   
   msg = g_dbus_message_new_signal ("/org/gtk/gio/DesktopAppInfo",
 				   "org.gtk.gio.DesktopAppInfo",
 				   "Launched");
   g_dbus_message_set_body (msg, g_variant_new ("(@aysxasa{sv})",
-					       g_variant_new_bytestring (desktop_file),
+					       g_variant_new_bytestring (desktop_file_id),
 					       display ? display : "",
 					       (gint64)pid,
 					       &uri_variant,
@@ -1069,7 +1091,7 @@ _g_desktop_app_info_launch_uris_internal (GAppInfo                   *appinfo,
 	pid_callback (info, pid, pid_callback_data);
 
       notify_desktop_launch (session_bus,
-			     info->filename,
+			     info,
 			     pid,
 			     data.display,
 			     data.sn_id,
@@ -1088,7 +1110,14 @@ _g_desktop_app_info_launch_uris_internal (GAppInfo                   *appinfo,
    * after launching an app.  See http://bugzilla.gnome.org/606960
    */
   if (session_bus != NULL)
-    g_object_unref (session_bus);
+    {
+      /* This asynchronous flush holds a reference until it completes,
+       * which ensures that the following unref won't immediately kill
+       * the connection if we were the initial owner.
+       */
+      g_dbus_connection_flush (session_bus, NULL, NULL, NULL);
+      g_object_unref (session_bus);
+    }
 
   completed = TRUE;
 
@@ -3030,14 +3059,19 @@ get_all_desktop_entries_for_mime_type (const char  *base_mime_type,
 	{
 	  dir = dir_list->data;
 
-          /* Pick the explicit default application */
-          entry = g_hash_table_lookup (dir->mimeapps_list_defaults_map, mime_type);
-
-          if (entry != NULL)
+          /* Pick the explicit default application if we got no result earlier
+           * (ie, for more specific mime types)
+           */
+          if (desktop_entries == NULL)
             {
-              /* Save the default entry if it's the first one we encounter */
-              if (default_entry == NULL)
-                default_entry = g_strdup (entry);
+              entry = g_hash_table_lookup (dir->mimeapps_list_defaults_map, mime_type);
+
+              if (entry != NULL)
+                {
+                  /* Save the default entry if it's the first one we encounter */
+                  if (default_entry == NULL)
+                    default_entry = g_strdup (entry);
+                }
             }
 
 	  /* Then added associations from mimeapps.list */
