@@ -20,6 +20,7 @@
  * Author: David Zeuthen <davidz@redhat.com>
  */
 
+#include <locale.h>
 #include <gio/gio.h>
 
 #include <string.h>
@@ -405,6 +406,18 @@ dbus_1_message_append (GString *s,
         break;
       }
 
+#ifdef DBUS_TYPE_UNIX_FD
+    case DBUS_TYPE_UNIX_FD:
+      {
+        /* unfortunately there's currently no way to get just the
+         * protocol value, since dbus_message_iter_get_basic() wants
+         * to be 'helpful' and dup the fd for the user...
+         */
+        g_string_append (s, "unix-fd: (not extracted)\n");
+        break;
+      }
+#endif
+
      case DBUS_TYPE_VARIANT:
        g_string_append_printf (s, "variant:\n");
        dbus_message_iter_recurse (iter, &sub);
@@ -484,12 +497,19 @@ get_body_signature (GVariant *value)
   gsize len;
   gchar *ret;
 
+  if (value == NULL)
+    {
+      ret = g_strdup ("");
+      goto out;
+    }
+
   s = g_variant_get_type_string (value);
   len = strlen (s);
-  g_assert (len>=2);
+  g_assert (len >= 2);
 
   ret = g_strndup (s + 1, len - 2);
 
+ out:
   return ret;
 }
 
@@ -506,6 +526,7 @@ check_serialization (GVariant *value,
   DBusError dbus_error;
   gchar *s;
   gchar *s1;
+  guint n;
 
   message = g_dbus_message_new ();
   g_dbus_message_set_body (message, value);
@@ -517,71 +538,108 @@ check_serialization (GVariant *value,
   g_dbus_message_set_header (message, G_DBUS_MESSAGE_HEADER_FIELD_SIGNATURE, g_variant_new_signature (s));
   g_free (s);
 
-  /* First check that the serialization to the D-Bus wire format is correct */
-
-  error = NULL;
-  blob = g_dbus_message_to_blob (message,
-                                 &blob_size,
-                                 G_DBUS_CAPABILITY_FLAGS_NONE,
-                                 &error);
-  g_assert_no_error (error);
-  g_assert (blob != NULL);
-
-  dbus_error_init (&dbus_error);
-  dbus_1_message = dbus_message_demarshal ((char *) blob, blob_size, &dbus_error);
-  if (dbus_error_is_set (&dbus_error))
+  /* First check that the serialization to the D-Bus wire format is correct - do this for both byte orders */
+  for (n = 0; n < 2; n++)
     {
-      g_printerr ("Error calling dbus_message_demarshal() on this blob: %s: %s\n",
-                  dbus_error.name,
-                  dbus_error.message);
-      hexdump (blob, blob_size);
-      dbus_error_free (&dbus_error);
+      GDBusMessageByteOrder byte_order;
+      switch (n)
+        {
+        case 0:
+          byte_order = G_DBUS_MESSAGE_BYTE_ORDER_BIG_ENDIAN;
+          break;
+        case 1:
+          byte_order = G_DBUS_MESSAGE_BYTE_ORDER_LITTLE_ENDIAN;
+          break;
+        case 2:
+          g_assert_not_reached ();
+          break;
+        }
+      g_dbus_message_set_byte_order (message, byte_order);
 
-      s = g_variant_print (value, TRUE);
-      g_printerr ("\nThe blob was generated from the following GVariant value:\n%s\n\n", s);
+      error = NULL;
+      blob = g_dbus_message_to_blob (message,
+                                     &blob_size,
+                                     G_DBUS_CAPABILITY_FLAGS_NONE,
+                                     &error);
+      g_assert_no_error (error);
+      g_assert (blob != NULL);
+
+      switch (byte_order)
+        {
+        case G_DBUS_MESSAGE_BYTE_ORDER_BIG_ENDIAN:
+          g_assert_cmpint (blob[0], ==, 'B');
+          break;
+        case G_DBUS_MESSAGE_BYTE_ORDER_LITTLE_ENDIAN:
+          g_assert_cmpint (blob[0], ==, 'l');
+          break;
+        }
+
+      dbus_error_init (&dbus_error);
+      dbus_1_message = dbus_message_demarshal ((char *) blob, blob_size, &dbus_error);
+      if (dbus_error_is_set (&dbus_error))
+        {
+          g_printerr ("Error calling dbus_message_demarshal() on this blob: %s: %s\n",
+                      dbus_error.name,
+                      dbus_error.message);
+          hexdump (blob, blob_size);
+          dbus_error_free (&dbus_error);
+
+          s = g_variant_print (value, TRUE);
+          g_printerr ("\nThe blob was generated from the following GVariant value:\n%s\n\n", s);
+          g_free (s);
+
+          g_printerr ("If the blob was encoded using DBusMessageIter, the payload would have been:\n");
+          print_gv_dbus_message (value);
+
+          g_assert_not_reached ();
+        }
+
+      s = dbus_1_message_print (dbus_1_message);
+      dbus_message_unref (dbus_1_message);
+
+      g_assert_cmpstr (s, ==, expected_dbus_1_output);
       g_free (s);
 
-      g_printerr ("If the blob was encoded using DBusMessageIter, the payload would have been:\n");
-      print_gv_dbus_message (value);
+      /* Then serialize back and check that the body is identical */
 
-      g_assert_not_reached ();
+      error = NULL;
+      recovered_message = g_dbus_message_new_from_blob (blob,
+                                                        blob_size,
+                                                        G_DBUS_CAPABILITY_FLAGS_NONE,
+                                                        &error);
+      g_assert (recovered_message != NULL);
+      g_assert_no_error (error);
+
+      if (value == NULL)
+        {
+          g_assert (g_dbus_message_get_body (recovered_message) == NULL);
+        }
+      else
+        {
+          g_assert (g_dbus_message_get_body (recovered_message) != NULL);
+          if (!g_variant_equal (g_dbus_message_get_body (recovered_message), value))
+            {
+              s = g_variant_print (g_dbus_message_get_body (recovered_message), TRUE);
+              s1 = g_variant_print (value, TRUE);
+              g_printerr ("Recovered value:\n%s\ndoes not match given value\n%s\n",
+                          s,
+                          s1);
+              g_free (s);
+              g_free (s1);
+              g_assert_not_reached ();
+            }
+        }
+      g_object_unref (recovered_message);
     }
 
-  s = dbus_1_message_print (dbus_1_message);
-  dbus_message_unref (dbus_1_message);
-
-  g_assert_cmpstr (s, ==, expected_dbus_1_output);
-  g_free (s);
-
-  /* Then serialize back and check that the body is identical */
-
-  error = NULL;
-  recovered_message = g_dbus_message_new_from_blob (blob,
-                                                    blob_size,
-                                                    G_DBUS_CAPABILITY_FLAGS_NONE,
-                                                    &error);
-  g_assert_no_error (error);
-  g_assert (recovered_message != NULL);
-  g_assert (g_dbus_message_get_body (recovered_message) != NULL);
-
-  if (!g_variant_equal (g_dbus_message_get_body (recovered_message), value))
-    {
-      s = g_variant_print (g_dbus_message_get_body (recovered_message), TRUE);
-      s1 = g_variant_print (value, TRUE);
-      g_printerr ("Recovered value:\n%s\ndoes not match given value\n%s\n",
-                  s,
-                  s1);
-      g_free (s);
-      g_free (s1);
-      g_assert_not_reached ();
-    }
   g_object_unref (message);
-  g_object_unref (recovered_message);
 }
 
 static void
 message_serialize_basic (void)
 {
+  check_serialization (NULL, "");
+
   check_serialization (g_variant_new ("(sogybnqiuxtd)",
                                       "this is a string",
                                       "/this/is/a/path",
@@ -656,20 +714,278 @@ message_serialize_complex (void)
                        "      string: `cwd'\n"
                        "      variant:\n"
                        "        string: `/home/davidz/Hacking/glib/gio/tests'\n");
+
+#ifdef DBUS_TYPE_UNIX_FD
+  value = g_variant_parse (G_VARIANT_TYPE ("(hah)"),
+                           "(42, [43, 44])",
+                           NULL, NULL, &error);
+  g_assert_no_error (error);
+  g_assert (value != NULL);
+  /* about (not extracted), see comment in DBUS_TYPE_UNIX_FD case in
+   * dbus_1_message_append() above.
+   */
+  check_serialization (value,
+                       "value 0:   unix-fd: (not extracted)\n"
+                       "value 1:   array:\n"
+                       "    unix-fd: (not extracted)\n"
+                       "    unix-fd: (not extracted)\n");
+#endif
 }
 
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+static void
+message_serialize_invalid (void)
+{
+  guint n;
+
+  /* Here we're relying on libdbus-1's DBusMessage type not checking
+   * anything. If that were to change, we'd need to do our own
+   * thing.
+   *
+   * Other things we could check (note that GDBus _does_ check for all
+   * these things - we just don't have test-suit coverage for it)
+   *
+   *  - array exceeding 64 MiB (2^26 bytes) - unfortunately libdbus-1 checks
+   *    this, e.g.
+   *
+   *      process 19620: arguments to dbus_message_iter_append_fixed_array() were incorrect,
+   *      assertion "n_elements <= DBUS_MAXIMUM_ARRAY_LENGTH / _dbus_type_get_alignment (element_type)"
+   *      failed in file dbus-message.c line 2344.
+   *      This is normally a bug in some application using the D-Bus library.
+   *      D-Bus not built with -rdynamic so unable to print a backtrace
+   *      Aborted (core dumped)
+   *
+   *  - message exceeding 128 MiB (2^27 bytes)
+   *
+   *  - endianness, message type, flags, protocol version
+   */
+
+  for (n = 0; n < 3; n++)
+    {
+      GDBusMessage *message;
+      GError *error;
+      DBusMessage *dbus_message;
+      char *blob;
+      int blob_len;
+      const gchar *invalid_utf8_str = "this is invalid\xff";
+      const gchar *invalid_object_path = "/this/is/not a valid object path";
+      const gchar *invalid_signature = "not valid signature";
+
+      dbus_message = dbus_message_new (DBUS_MESSAGE_TYPE_METHOD_CALL);
+      dbus_message_set_serial (dbus_message, 0x41);
+      dbus_message_set_path (dbus_message, "/foo/bar");
+      dbus_message_set_member (dbus_message, "Member");
+      switch (n)
+        {
+        case 0:
+          /* invalid UTF-8 */
+          dbus_message_append_args (dbus_message,
+                                    DBUS_TYPE_STRING, &invalid_utf8_str,
+                                    DBUS_TYPE_INVALID);
+          break;
+
+        case 1:
+          /* invalid object path */
+          dbus_message_append_args (dbus_message,
+                                    DBUS_TYPE_OBJECT_PATH, &invalid_object_path,
+                                    DBUS_TYPE_INVALID);
+          break;
+
+        case 2:
+          /* invalid signature */
+          dbus_message_append_args (dbus_message,
+                                    DBUS_TYPE_SIGNATURE, &invalid_signature,
+                                    DBUS_TYPE_INVALID);
+          break;
+
+        default:
+          g_assert_not_reached ();
+          break;
+        }
+      dbus_message_marshal (dbus_message, &blob, &blob_len);
+
+      error = NULL;
+      message = g_dbus_message_new_from_blob ((guchar *) blob,
+                                              blob_len,
+                                              G_DBUS_CAPABILITY_FLAGS_NONE,
+                                              &error);
+      g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+      g_error_free (error);
+      g_assert (message == NULL);
+
+      dbus_free (blob);
+    }
+
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+static void
+message_serialize_header_checks (void)
+{
+  GDBusMessage *message;
+  GDBusMessage *reply;
+  GError *error;
+  guchar *blob;
+  gsize blob_size;
+
+  /*
+   * check we can't serialize messages with INVALID type
+   */
+  message = g_dbus_message_new ();
+  error = NULL;
+  blob = g_dbus_message_to_blob (message, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: type is INVALID");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  g_object_unref (message);
+
+  /*
+   * check we can't serialize signal messages with INTERFACE, PATH or MEMBER unset / set to reserved value
+   */
+  message = g_dbus_message_new_signal ("/the/path", "The.Interface", "TheMember");
+  /* ----- */
+  /* interface NULL => error */
+  g_dbus_message_set_interface (message, NULL);
+  error = NULL;
+  blob = g_dbus_message_to_blob (message, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: SIGNAL message: PATH, INTERFACE or MEMBER header field is missing");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  /* interface reserved value => error */
+  g_dbus_message_set_interface (message, "org.freedesktop.DBus.Local");
+  error = NULL;
+  blob = g_dbus_message_to_blob (message, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: SIGNAL message: The INTERFACE header field is using the reserved value org.freedesktop.DBus.Local");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  /* reset interface */
+  g_dbus_message_set_interface (message, "The.Interface");
+  /* ----- */
+  /* path NULL => error */
+  g_dbus_message_set_path (message, NULL);
+  error = NULL;
+  blob = g_dbus_message_to_blob (message, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: SIGNAL message: PATH, INTERFACE or MEMBER header field is missing");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  /* path reserved value => error */
+  g_dbus_message_set_path (message, "/org/freedesktop/DBus/Local");
+  error = NULL;
+  blob = g_dbus_message_to_blob (message, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: SIGNAL message: The PATH header field is using the reserved value /org/freedesktop/DBus/Local");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  /* reset path */
+  g_dbus_message_set_path (message, "/the/path");
+  /* ----- */
+  /* member NULL => error */
+  g_dbus_message_set_member (message, NULL);
+  error = NULL;
+  blob = g_dbus_message_to_blob (message, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: SIGNAL message: PATH, INTERFACE or MEMBER header field is missing");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  /* reset member */
+  g_dbus_message_set_member (message, "TheMember");
+  /* ----- */
+  /* done */
+  g_object_unref (message);
+
+  /*
+   * check that we can't serialize method call messages with PATH or MEMBER unset
+   */
+  message = g_dbus_message_new_method_call (NULL, "/the/path", NULL, "TheMember");
+  /* ----- */
+  /* path NULL => error */
+  g_dbus_message_set_path (message, NULL);
+  error = NULL;
+  blob = g_dbus_message_to_blob (message, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: METHOD_CALL message: PATH or MEMBER header field is missing");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  /* reset path */
+  g_dbus_message_set_path (message, "/the/path");
+  /* ----- */
+  /* member NULL => error */
+  g_dbus_message_set_member (message, NULL);
+  error = NULL;
+  blob = g_dbus_message_to_blob (message, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: METHOD_CALL message: PATH or MEMBER header field is missing");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  /* reset member */
+  g_dbus_message_set_member (message, "TheMember");
+  /* ----- */
+  /* done */
+  g_object_unref (message);
+
+  /*
+   * check that we can't serialize method reply messages with REPLY_SERIAL unset
+   */
+  message = g_dbus_message_new_method_call (NULL, "/the/path", NULL, "TheMember");
+  g_dbus_message_set_serial (message, 42);
+  /* method reply */
+  reply = g_dbus_message_new_method_reply (message);
+  g_assert_cmpint (g_dbus_message_get_reply_serial (reply), ==, 42);
+  g_dbus_message_set_header (reply, G_DBUS_MESSAGE_HEADER_FIELD_REPLY_SERIAL, NULL);
+  error = NULL;
+  blob = g_dbus_message_to_blob (reply, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: METHOD_RETURN message: REPLY_SERIAL header field is missing");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  g_object_unref (reply);
+  /* method error - first nuke ERROR_NAME, then REPLY_SERIAL */
+  reply = g_dbus_message_new_method_error (message, "Some.Error.Name", "the message");
+  g_assert_cmpint (g_dbus_message_get_reply_serial (reply), ==, 42);
+  /* nuke ERROR_NAME */
+  g_dbus_message_set_error_name (reply, NULL);
+  error = NULL;
+  blob = g_dbus_message_to_blob (reply, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: ERROR message: REPLY_SERIAL or ERROR_NAME header field is missing");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  /* reset ERROR_NAME */
+  g_dbus_message_set_error_name (reply, "Some.Error.Name");
+  /* nuke REPLY_SERIAL */
+  g_dbus_message_set_header (reply, G_DBUS_MESSAGE_HEADER_FIELD_REPLY_SERIAL, NULL);
+  error = NULL;
+  blob = g_dbus_message_to_blob (reply, &blob_size, G_DBUS_CAPABILITY_FLAGS_NONE, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert_cmpstr (error->message, ==, "Cannot serialize message: ERROR message: REPLY_SERIAL or ERROR_NAME header field is missing");
+  g_error_free (error);
+  g_assert (blob == NULL);
+  g_object_unref (reply);
+  g_object_unref (message);
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 int
 main (int   argc,
       char *argv[])
 {
+  setlocale (LC_ALL, "C");
+
   g_type_init ();
   g_test_init (&argc, &argv, NULL);
 
   g_test_add_func ("/gdbus/message-serialize-basic", message_serialize_basic);
   g_test_add_func ("/gdbus/message-serialize-complex", message_serialize_complex);
+  g_test_add_func ("/gdbus/message-serialize-invalid", message_serialize_invalid);
+  g_test_add_func ("/gdbus/message-serialize-header-checks", message_serialize_header_checks);
   return g_test_run();
 }
 
